/* qdbusobject.h DBUS service object interface
 *
 * Copyright (C) 2005 Kevin Krammer <kevin.krammer@gmx.at>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 *
 */

#ifndef QDBUSOBJECT_H
#define QDBUSOBJECT_H

/**
 * @page dbusservice Providing services over DBus
 *
 * The Qt3 bindings do not support autogeneration of service objects yet. In
 * order to provide interfaces over DBus, an application has to implement the
 * QDBusObjectBase interface and register an instance of the resulting class
 * with the QDBusConnection.
 *
 *
 * @section dbusservice-example A simple DBus client example
 *
 * @code
 *   #include <dbus/qdbusconnection.h>;
 *   #include <dbus/qdbusobject.h>;
 *
 *   class QStringList;
 *
 *   class TestService : public QDBusObjectBase
 *   {
 *   public:
 *       TestService(const QDBusConnection& connection);
 *       virtual ~TestService();
 *
 *   protected:
 *       virtual bool handleMethodCall(const QDBusMessage& message);
 *
 *   private:
 *       QDBusConnection m_connection;
 *
 *   private:
 *       QStringList sortStrings(const QStringList& list);
 *   };
 * @endcode
 * @code
 *
 *   #include <qstringlist.h>;
 *
 *   #include <dbus/qdbuserror.h>;
 *   #include <dbus/qdbusmessage.h>;
 *
 *   TestService::TestService(const QDBusConnection& connection) : m_connection(connection)
 *   {
 *       m_connection.registerObject("/ListSorter", this);
 *   }
 *
 *   TestService::~TestService()
 *   {
 *       m_connection.unregisterObject("/ListSorter");
 *   }
 *
 *   // return false to let D-BUS send a standard error message that the method is unknown
 *
 *   bool TestService::handleMethod(const QDBusMessage& message)
 *   {
 *       if (message.interface() != "org.example.Sort") return false;
 *
 *       if (message.member() == "Strings")
 *       {
 *           // check parameters
 *
 *           if (message.count() != 1 || message[0].type() != QVariant::StringList)
 *           {
 *               // method signature not what we expected
 *
 *               QDBusError error("org.freedesktop.DBus.Error.InvalidSignature",
 *                                "Expected one argument of type string list");
 *
 *               QDBusMessage reply = QDBusMessage::methodError(message, error);
 *
 *               // send error
 *
 *               m_connection.send(reply);
 *
 *               // tell D-BUS we did handle the call
 *
 *               return true;
 *           }
 *
 *           // call implementation
 *
 *           QStringList result = sortStrings(message[0].toStringList());
 *
 *           // prepare reply
 *
 *           QDBusMessage reply = QDBusMessage::methodReply(message);
 *
 *           reply << result;
 *
 *           // send reply
 *
 *           m_connection.send(reply);
 *
 *           // tell D-BUS we did handle the call
 *
 *           return true;
 *       }
 *
 *       return false;
 *   }
 *
 *   QStringList TestService::sortStrings(const QStringList& list)
 *   {
 *       QStringList result = list;
 *
 *       result.sort();
 *
 *       return result;
 *   }
 * @endcode
 */

/**
 * @include example-service.h
 * @example example-service.cpp
 */

#include "dbus/qdbusmacros.h"

class QDBusMessage;

/**
 * @brief Base interface for DBus service objects
 *
 * In order to register a service object with the QDBusConnection it needs to
 * implement the interface specified by this class.
 *
 * The connection will forward all method calls that have a path equivalent
 * to the path the service object was registered with to the object's
 * handleMethodCall() method. See QDBusConnection::registerObject()
 *
 * If for some reason, e.g. the call is not meant for this interface, or the
 * method is unknown, the implementation can just return @c false and the
 * connection will handle the rest.
 *
 * See section @ref dbusservice for documentation on how to use QDBusObjectBase
 */
class QDBUS_EXPORT QDBusObjectBase
{
    friend class QDBusConnectionPrivate;
public:
    /**
     * @brief Destroys the object
     */
    virtual ~QDBusObjectBase() {}

protected:
    /**
     * @brief Method call entry point
     *
     * This method has to be implemented to handle method calls sent to the
     * service object.
     * An object implementation can handle all its interfaces in one class or
     * again forward the method call to interface implementators.
     *
     * If for some reason, e.g. the call is not meant for this interface, or
     * the method is unknown, the implementation can just return @c false and
     * the connection will handle the rest.
     *
     * If an error occurs during the method call, e.g. the number of parameters
     * or their types are not what would be expected, the service object
     * should reply with a QDBusMessage of type QDBusMessage::ErrorMessage
     * which in turn should include the DBus error describing the problem.
     * See QDBusConnection::send() for sending reply messages.
     *
     * See QDBusMessage::methodError() and QDBusMessage::methodReply() on
     * how to create suitable reply messages for the given method call.
     *
     * @param message the method call to handle
     *
     * @return @c true if the message can be handled independent if handling
     *         resulted in an error. In this case implementations should an
     *         error reply. Returns @c false only if interface or method are
     *         unknown
     */
    virtual bool handleMethodCall(const QDBusMessage& message) = 0;
};

#endif

